@@ -6,6 +6,7 @@
 
 use std::time::Duration;
 
+use crate::main::PixelSize;
 use crate::outputs::OutputId;
 
 /// The keyboard should hide after this has elapsed to prevent flickering.
@@ -16,7 +17,7 @@ pub const HIDING_TIMEOUT: Duration = Duration::from_millis(200);
 pub enum Outcome {
     Visible {
         output: OutputId,
-        height: u32,
+        height: PixelSize,
     },
     Hidden,
 }
