@@ -1,17 +1,17 @@
 /*!
  * Layout-related data.
- * 
+ *
  * The `View` contains `Row`s and each `Row` contains `Button`s.
  * They carry data relevant to their positioning only,
  * except the Button, which also carries some data
  * about its appearance and function.
- * 
+ *
  * The layout is determined bottom-up, by measuring `Button` sizes,
  * deriving `Row` sizes from them, and then centering them within the `View`.
- * 
+ *
  * That makes the `View` position immutable,
  * and therefore different than the other positions.
- * 
+ *
  * Note that it might be a better idea
  * to make `View` position depend on its contents,
  * and let the renderer scale and center it within the widget.
@@ -32,6 +32,8 @@ use ::manager;
 use ::submission::{ Submission, SubmitData, Timestamp };
 use ::util::find_max_double;
 
+use ::imservice::ContentPurpose;
+
 // Traits
 use std::borrow::Borrow;
 use ::logging::Warn;
@@ -41,9 +43,8 @@ pub mod c {
     use super::*;
 
     use gtk_sys;
-    use std::ffi::CStr;
-    use std::os::raw::{ c_char, c_void };
-    use std::ptr;
+    use std::os::raw::c_void;
+    use crate::submission::c::Submission as CSubmission;
 
     use std::ops::{ Add, Sub };
 
@@ -52,7 +53,6 @@ pub mod c {
     #[derive(Copy, Clone)]
     pub struct EekGtkKeyboard(pub *const gtk_sys::GtkWidget);
 
-    #[no_mangle]
     extern "C" {
         #[allow(improper_ctypes)]
         pub fn eek_gtk_keyboard_emit_feedback(
@@ -67,7 +67,7 @@ pub mod c {
         pub x: f64,
         pub y: f64,
     }
-    
+
     impl Add for Point {
         type Output = Self;
         fn add(self, other: Self) -> Self {
@@ -84,7 +84,7 @@ pub mod c {
             }
         }
     }
-    
+
     impl Sub<&Point> for Point {
         type Output = Point;
         fn sub(self, other: &Point) -> Point {
@@ -155,7 +155,7 @@ pub mod c {
             }
         }
     }
-    
+
     // This is constructed only in C, no need for warnings
     #[allow(dead_code)]
     #[repr(transparent)]
@@ -163,64 +163,6 @@ pub mod c {
 
     // The following defined in Rust. TODO: wrap naked pointers to Rust data inside RefCells to prevent multiple writers
 
-    #[no_mangle]
-    pub extern "C"
-    fn squeek_button_get_bounds(button: *const ::layout::Button) -> Bounds {
-        let button = unsafe { &*button };
-        Bounds {
-            x: 0.0, y: 0.0,
-            width: button.size.width, height: button.size.height
-        }
-    }
-
-    #[no_mangle]
-    pub extern "C"
-    fn squeek_button_get_label(
-        button: *const ::layout::Button
-    ) -> *const c_char {
-        let button = unsafe { &*button };
-        match &button.label {
-            Label::Text(text) => text.as_ptr(),
-            // returning static strings to C is a bit cumbersome
-            Label::IconName(_) => unsafe {
-                // CStr doesn't allocate anything, so it only points to
-                // the 'static str, avoiding a memory leak
-                CStr::from_bytes_with_nul_unchecked(b"icon\0")
-            }.as_ptr(),
-        }
-    }
-    
-    #[no_mangle]
-    pub extern "C"
-    fn squeek_button_get_icon_name(button: *const Button) -> *const c_char {
-        let button = unsafe { &*button };
-        match &button.label {
-            Label::Text(_) => ptr::null(),
-            Label::IconName(name) => name.as_ptr(),
-        }
-    }
-    
-    #[no_mangle]
-    pub extern "C"
-    fn squeek_button_get_name(button: *const Button) -> *const c_char {
-        let button = unsafe { &*button };
-        button.name.as_ptr()
-    }
-    
-    #[no_mangle]
-    pub extern "C"
-    fn squeek_button_get_outline_name(button: *const Button) -> *const c_char {
-        let button = unsafe { &*button };
-        button.outline_name.as_ptr()
-    }
-    
-    #[no_mangle]
-    pub extern "C"
-    fn squeek_button_print(button: *const ::layout::Button) {
-        let button = unsafe { &*button };
-        println!("{:?}", button);
-    }
-    
     /// Positions the layout contents within the available space.
     /// The origin of the transformation is the point inside the margins.
     #[no_mangle]
@@ -244,6 +186,13 @@ pub mod c {
         layout.kind.clone() as u32
     }
 
+    #[no_mangle]
+    pub extern "C"
+    fn squeek_layout_get_purpose(layout: *const Layout) -> u32 {
+        let layout = unsafe { &*layout };
+        layout.purpose.clone() as u32
+    }
+
     #[no_mangle]
     pub extern "C"
     fn squeek_layout_free(layout: *mut Layout) {
@@ -259,7 +208,7 @@ pub mod c {
         pub extern "C"
         fn squeek_layout_release(
             layout: *mut Layout,
-            submission: *mut Submission,
+            submission: CSubmission,
             widget_to_layout: Transformation,
             time: u32,
             manager: manager::c::Manager,
@@ -267,7 +216,8 @@ pub mod c {
         ) {
             let time = Timestamp(time);
             let layout = unsafe { &mut *layout };
-            let submission = unsafe { &mut *submission };
+            let submission = submission.clone_ref();
+            let mut submission = submission.borrow_mut();
             let ui_backend = UIBackend {
                 widget_to_layout,
                 keyboard: ui_keyboard,
@@ -279,7 +229,7 @@ pub mod c {
                 let key: &Rc<RefCell<KeyState>> = key.borrow();
                 seat::handle_release_key(
                     layout,
-                    submission,
+                    &mut submission,
                     Some(&ui_backend),
                     time,
                     Some(manager),
@@ -294,18 +244,19 @@ pub mod c {
         pub extern "C"
         fn squeek_layout_release_all_only(
             layout: *mut Layout,
-            submission: *mut Submission,
+            submission: CSubmission,
             time: u32,
         ) {
             let layout = unsafe { &mut *layout };
-            let submission = unsafe { &mut *submission };
+            let submission = submission.clone_ref();
+            let mut submission = submission.borrow_mut();
             // The list must be copied,
             // because it will be mutated in the loop
             for key in layout.pressed_keys.clone() {
                 let key: &Rc<RefCell<KeyState>> = key.borrow();
                 seat::handle_release_key(
                     layout,
-                    submission,
+                    &mut submission,
                     None, // don't update UI
                     Timestamp(time),
                     None, // don't switch layouts
@@ -318,25 +269,26 @@ pub mod c {
         pub extern "C"
         fn squeek_layout_depress(
             layout: *mut Layout,
-            submission: *mut Submission,
+            submission: CSubmission,
             x_widget: f64, y_widget: f64,
             widget_to_layout: Transformation,
             time: u32,
             ui_keyboard: EekGtkKeyboard,
         ) {
             let layout = unsafe { &mut *layout };
-            let submission = unsafe { &mut *submission };
+            let submission = submission.clone_ref();
+            let mut submission = submission.borrow_mut();
             let point = widget_to_layout.forward(
                 Point { x: x_widget, y: y_widget }
             );
 
             let state = layout.find_button_by_position(point)
                 .map(|place| place.button.state.clone());
-            
+
             if let Some(state) = state {
                 seat::handle_press_key(
                     layout,
-                    submission,
+                    &mut submission,
                     Timestamp(time),
                     &state,
                 );
@@ -355,7 +307,7 @@ pub mod c {
         pub extern "C"
         fn squeek_layout_drag(
             layout: *mut Layout,
-            submission: *mut Submission,
+            submission: CSubmission,
             x_widget: f64, y_widget: f64,
             widget_to_layout: Transformation,
             time: u32,
@@ -364,7 +316,8 @@ pub mod c {
         ) {
             let time = Timestamp(time);
             let layout = unsafe { &mut *layout };
-            let submission = unsafe { &mut *submission };
+            let submission = submission.clone_ref();
+            let mut submission = submission.borrow_mut();
             let ui_backend = UIBackend {
                 widget_to_layout,
                 keyboard: ui_keyboard,
@@ -372,7 +325,7 @@ pub mod c {
             let point = ui_backend.widget_to_layout.forward(
                 Point { x: x_widget, y: y_widget }
             );
-            
+
             let pressed = layout.pressed_keys.clone();
             let button_info = {
                 let place = layout.find_button_by_position(point);
@@ -392,7 +345,7 @@ pub mod c {
                     } else {
                         seat::handle_release_key(
                             layout,
-                            submission,
+                            &mut submission,
                             Some(&ui_backend),
                             time,
                             Some(manager),
@@ -403,7 +356,7 @@ pub mod c {
                 if !found {
                     seat::handle_press_key(
                         layout,
-                        submission,
+                        &mut submission,
                         time,
                         &state,
                     );
@@ -417,7 +370,7 @@ pub mod c {
                     let key: &Rc<RefCell<KeyState>> = wrapped_key.borrow();
                     seat::handle_release_key(
                         layout,
-                        submission,
+                        &mut submission,
                         Some(&ui_backend),
                         time,
                         Some(manager),
@@ -431,11 +384,11 @@ pub mod c {
         #[cfg(test)]
         mod test {
             use super::*;
-            
+
             fn near(a: f64, b: f64) -> bool {
                 (a - b).abs() < ((a + b) * 0.001f64).abs()
             }
-            
+
             #[test]
             fn transform_back() {
                 let transform = Transformation {
@@ -474,7 +427,7 @@ pub enum Label {
 /// The graphical representation of a button
 #[derive(Clone, Debug)]
 pub struct Button {
-    /// ID string, e.g. for CSS 
+    /// ID string, e.g. for CSS
     pub name: CString,
     /// Label to display to the user
     pub label: Label,
@@ -485,6 +438,15 @@ pub struct Button {
     pub state: Rc<RefCell<KeyState>>,
 }
 
+impl Button {
+    pub fn get_bounds(&self) -> c::Bounds {
+        c::Bounds {
+            x: 0.0, y: 0.0,
+            width: self.size.width, height: self.size.height,
+        }
+    }
+}
+
 /// The graphical representation of a row of buttons
 #[derive(Clone, Debug)]
 pub struct Row {
@@ -552,6 +514,7 @@ pub struct Spacing {
     pub button: f64,
 }
 
+#[derive(Clone)]
 pub struct View {
     /// Rows together with their offsets from the top left
     rows: Vec<(c::Point, Row)>,
@@ -624,11 +587,11 @@ impl View {
             offset: &row.0 + c::Point { x: button.0, y: 0.0 },
         })
     }
-    
+
     pub fn get_size(&self) -> Size {
         self.size.clone()
     }
-    
+
     /// Returns positioned rows, with appropriate x offsets (centered)
     pub fn get_rows(&self) -> &Vec<(c::Point, Row)> {
         &self.rows
@@ -651,7 +614,7 @@ impl View {
 }
 
 /// The physical characteristic of layout for the purpose of styling
-#[derive(Clone, PartialEq, Debug)]
+#[derive(Clone, Copy, PartialEq, Debug)]
 pub enum ArrangementKind {
     Base = 0,
     Wide = 1,
@@ -665,13 +628,27 @@ pub struct Margins {
     pub right: f64,
 }
 
+#[derive(Clone, Debug, PartialEq)]
+pub enum LatchedState {
+    /// Holds view to return to.
+    FromView(String),
+    Not,
+}
+
 // TODO: split into sth like
 // Arrangement (views) + details (keymap) + State (keys)
 /// State of the UI, contains the backend as well
 pub struct Layout {
     pub margins: Margins,
     pub kind: ArrangementKind,
+    pub purpose: ContentPurpose,
     pub current_view: String,
+
+    // If current view is latched,
+    // clicking any button that emits an action (erase, submit, set modifier)
+    // will cause lock buttons to unlatch.
+    view_latched: LatchedState,
+
     // Views own the actual buttons which have state
     // Maybe they should own UI only,
     // and keys should be owned by a dedicated non-UI-State?
@@ -714,14 +691,16 @@ impl fmt::Display for NoSuchView {
 // The usage of &mut on Rc<RefCell<KeyState>> doesn't mean anything special.
 // Cloning could also be used.
 impl Layout {
-    pub fn new(data: LayoutData, kind: ArrangementKind) -> Layout {
+    pub fn new(data: LayoutData, kind: ArrangementKind, purpose: ContentPurpose) -> Layout {
         Layout {
             kind,
             current_view: "base".to_owned(),
+            view_latched: LatchedState::Not,
             views: data.views,
             keymaps: data.keymaps,
             pressed_keys: HashSet::new(),
             margins: data.margins,
+            purpose,
         }
     }
 
@@ -743,6 +722,12 @@ impl Layout {
         }
     }
 
+    // Layout is passed around mutably,
+    // so better keep the field away from direct access.
+    pub fn get_view_latched(&self) -> &LatchedState {
+        &self.view_latched
+    }
+
     /// Calculates size without margins
     fn calculate_inner_size(&self) -> Size {
         View::calculate_super_size(
@@ -762,7 +747,7 @@ impl Layout {
             ),
         }
     }
-    
+
     pub fn calculate_transformation(
         &self,
         available: Size,
@@ -802,24 +787,116 @@ impl Layout {
         }
     }
 
-    pub fn get_locked_keys(&self) -> Vec<Rc<RefCell<KeyState>>> {
-        let mut out = Vec::new();
-        let view = self.get_current_view();
-        for (_, row) in view.get_rows() {
-            for (_, button) in &row.buttons {
-                let locked = {
-                    let state = RefCell::borrow(&button.state).clone();
-                    state.action.is_locked(&self.current_view)
-                };
-                if locked {
-                    out.push(button.state.clone());
-                }
+    fn apply_view_transition(
+        &mut self,
+        action: &Action,
+    ) {
+        let (transition, new_latched) = Layout::process_action_for_view(
+            action,
+            &self.current_view,
+            &self.view_latched,
+        );
+
+        match transition {
+            ViewTransition::UnlatchAll => self.unstick_locks(),
+            ViewTransition::ChangeTo(view) => try_set_view(self, view.into()),
+            ViewTransition::NoChange => {},
+        };
+
+        self.view_latched = new_latched;
+    }
+
+    /// Unlatch all latched keys,
+    /// so that the new view is the one before first press.
+    fn unstick_locks(&mut self) {
+        if let LatchedState::FromView(name) = self.view_latched.clone() {
+            match self.set_view(name.clone()) {
+                Ok(_) => { self.view_latched = LatchedState::Not; }
+                Err(e) => log_print!(
+                    logging::Level::Bug,
+                    "Bad view {}, can't unlatch ({:?})",
+                    name,
+                    e,
+                ),
             }
         }
-        out
+    }
+
+    /// Last bool is new latch state.
+    /// It doesn't make sense when the result carries UnlatchAll,
+    /// but let's not be picky.
+    ///
+    /// Although the state is not defined at the keys
+    /// (it's in the relationship between view and action),
+    /// keys go through the following stages when clicked repeatedly:
+    /// unlocked+unlatched -> locked+latched -> locked+unlatched
+    /// -> unlocked+unlatched
+    fn process_action_for_view<'a>(
+        action: &'a Action,
+        current_view: &str,
+        latched: &LatchedState,
+    ) -> (ViewTransition<'a>, LatchedState) {
+        match action {
+            Action::Submit { text: _, keys: _ }
+                | Action::Erase
+                | Action::ApplyModifier(_)
+            => {
+                let t = match latched {
+                    LatchedState::FromView(_) => ViewTransition::UnlatchAll,
+                    LatchedState::Not => ViewTransition::NoChange,
+                };
+                (t, LatchedState::Not)
+            },
+            Action::SetView(view) => (
+                ViewTransition::ChangeTo(view),
+                LatchedState::Not,
+            ),
+            Action::LockView { lock, unlock, latches, looks_locked_from: _ } => {
+                use self::ViewTransition as VT;
+                let locked = action.is_locked(current_view);
+                match (locked, latched, latches) {
+                    // Was unlocked, now make locked but latched.
+                    (false, LatchedState::Not, true) => (
+                        VT::ChangeTo(lock),
+                        LatchedState::FromView(current_view.into()),
+                    ),
+                    // Layout is latched for reason other than this button.
+                    (false, LatchedState::FromView(view), true) => (
+                        VT::ChangeTo(lock),
+                        LatchedState::FromView(view.clone()),
+                    ),
+                    // Was latched, now only locked.
+                    (true, LatchedState::FromView(_), true)
+                        => (VT::NoChange, LatchedState::Not),
+                    // Was unlocked, can't latch so now make fully locked.
+                    (false, _, false)
+                        => (VT::ChangeTo(lock), LatchedState::Not),
+                    // Was locked, now make unlocked.
+                    (true, _, _)
+                        => (VT::ChangeTo(unlock), LatchedState::Not),
+                }
+            },
+            _ => (ViewTransition::NoChange, latched.clone()),
+        }
     }
 }
 
+#[derive(Debug, PartialEq)]
+enum ViewTransition<'a> {
+    ChangeTo(&'a str),
+    UnlatchAll,
+    NoChange,
+}
+
+fn try_set_view(layout: &mut Layout, view_name: &str) {
+    layout.set_view(view_name.into())
+        .or_print(
+            logging::Problem::Bug,
+            &format!("Bad view {}, ignoring", view_name),
+        );
+}
+
+
 mod procedures {
     use super::*;
 
@@ -845,7 +922,7 @@ mod procedures {
                 })
         }).collect()
     }
-    
+
     #[cfg(test)]
     mod test {
         use super::*;
@@ -898,67 +975,6 @@ pub struct UIBackend {
 mod seat {
     use super::*;
 
-    fn try_set_view(layout: &mut Layout, view_name: String) {
-        layout.set_view(view_name.clone())
-            .or_print(
-                logging::Problem::Bug,
-                &format!("Bad view {}, ignoring", view_name),
-            );
-    }
-
-    /// A vessel holding an obligation to switch view.
-    /// Use with #[must_use]
-    struct ViewChange<'a> {
-        layout: &'a mut Layout,
-        view_name: Option<String>,
-    }
-    
-    impl<'a> ViewChange<'a> {
-        fn choose_view(self, view_name: String) -> ViewChange<'a> {
-            ViewChange {
-                view_name: Some(view_name),
-                ..self
-            }
-        }
-        fn apply(self) {
-            if let Some(name) = self.view_name {
-                try_set_view(self.layout, name);
-            }
-        }
-    }
-
-    /// Find all impermanent view changes and undo them in an arbitrary order.
-    /// Return an obligation to actually switch the view.
-    /// The final view is the "unlock" view
-    /// from one of the currently stuck keys.
-    // As long as only one stuck button is used, this should be fine.
-    // This is guaranteed because pressing a lock button unlocks all others.
-    // TODO: Make some broader guarantee about the resulting view,
-    // e.g. by maintaining a stack of stuck keys.
-    #[must_use]
-    fn unstick_locks(layout: &mut Layout) -> ViewChange {
-        let mut new_view = None;
-        for key in layout.get_locked_keys().clone() {
-            let key: &Rc<RefCell<KeyState>> = key.borrow();
-            let key = RefCell::borrow(key);
-            match &key.action {
-                Action::LockView { lock: _, unlock: view } => {
-                    new_view = Some(view.clone());
-                },
-                a => log_print!(
-                    logging::Level::Bug,
-                    "Non-locking action {:?} was found inside locked keys",
-                    a,
-                ),
-            };
-        }
-        
-        ViewChange {
-            layout,
-            view_name: new_view,
-        }
-    }
-
     pub fn handle_press_key(
         layout: &mut Layout,
         submission: &mut Submission,
@@ -1018,37 +1034,22 @@ mod seat {
         };
         let action = key.action.clone();
 
+        layout.apply_view_transition(&action);
+
         // update
         let key = key.into_released();
 
-        // process changes
+        // process non-view switching
         match action {
             Action::Submit { text: _, keys: _ }
                 | Action::Erase
             => {
-                unstick_locks(layout).apply();
                 submission.handle_release(KeyState::get_id(rckey), time);
             },
-            Action::SetView(view) => {
-                try_set_view(layout, view)
-            },
-            Action::LockView { lock, unlock } => {
-                let gets_locked = !key.action.is_locked(&layout.current_view);
-                unstick_locks(layout)
-                    // It doesn't matter what the resulting view should be,
-                    // it's getting changed anyway.
-                    .choose_view(
-                        match gets_locked {
-                            true => lock.clone(),
-                            false => unlock.clone(),
-                        }
-                    )
-                    .apply()
-            },
             Action::ApplyModifier(modifier) => {
                 // FIXME: key id is unneeded with stateless locks
                 let key_id = KeyState::get_id(rckey);
-                let gets_locked = !submission.is_modifier_active(modifier.clone());
+                let gets_locked = !submission.is_modifier_active(modifier);
                 match gets_locked {
                     true => submission.handle_add_modifier(
                         key_id,
@@ -1083,6 +1084,8 @@ mod seat {
                     }
                 }
             },
+            // Other keys are handled in view switcher before.
+            _ => {}
         };
 
         let pointer = ::util::Pointer(rckey.clone());
@@ -1100,14 +1103,20 @@ mod test {
     use std::ffi::CString;
     use ::keyboard::PressType;
 
-    pub fn make_state() -> Rc<RefCell<::keyboard::KeyState>> {
+    pub fn make_state_with_action(action: Action)
+        -> Rc<RefCell<::keyboard::KeyState>>
+    {
         Rc::new(RefCell::new(::keyboard::KeyState {
             pressed: PressType::Released,
             keycodes: Vec::new(),
-            action: Action::SetView("default".into()),
+            action,
         }))
     }
 
+    pub fn make_state() -> Rc<RefCell<::keyboard::KeyState>> {
+        make_state_with_action(Action::SetView("default".into()))
+    }
+
     pub fn make_button_with_state(
         name: String,
         state: Rc<RefCell<::keyboard::KeyState>>,
@@ -1120,7 +1129,246 @@ mod test {
             state: state,
         })
     }
-    
+
+    #[test]
+    fn latch_lock_unlock() {
+        let action = Action::LockView {
+            lock: "lock".into(),
+            unlock: "unlock".into(),
+            latches: true,
+            looks_locked_from: vec![],
+        };
+
+        assert_eq!(
+            Layout::process_action_for_view(&action, "unlock", &LatchedState::Not),
+            (ViewTransition::ChangeTo("lock"), LatchedState::FromView("unlock".into())),
+        );
+
+        assert_eq!(
+            Layout::process_action_for_view(&action, "lock", &LatchedState::FromView("unlock".into())),
+            (ViewTransition::NoChange, LatchedState::Not),
+        );
+
+        assert_eq!(
+            Layout::process_action_for_view(&action, "lock", &LatchedState::Not),
+            (ViewTransition::ChangeTo("unlock"), LatchedState::Not),
+        );
+
+        assert_eq!(
+            Layout::process_action_for_view(&Action::Erase, "lock", &LatchedState::FromView("base".into())),
+            (ViewTransition::UnlatchAll, LatchedState::Not),
+        );
+    }
+
+    #[test]
+    fn latch_pop_layout() {
+        let switch = Action::LockView {
+            lock: "locked".into(),
+            unlock: "base".into(),
+            latches: true,
+            looks_locked_from: vec![],
+        };
+
+        let submit = Action::Erase;
+
+        let view = View::new(vec![(
+            0.0,
+            Row::new(vec![
+                (
+                    0.0,
+                    make_button_with_state(
+                        "switch".into(),
+                        make_state_with_action(switch.clone())
+                    ),
+                ),
+                (
+                    1.0,
+                    make_button_with_state(
+                        "submit".into(),
+                        make_state_with_action(submit.clone())
+                    ),
+                ),
+            ]),
+        )]);
+
+        let mut layout = Layout {
+            current_view: "base".into(),
+            view_latched: LatchedState::Not,
+            keymaps: Vec::new(),
+            kind: ArrangementKind::Base,
+            pressed_keys: HashSet::new(),
+            margins: Margins {
+                top: 0.0,
+                left: 0.0,
+                right: 0.0,
+                bottom: 0.0,
+            },
+            views: hashmap! {
+                // Both can use the same structure.
+                // Switching doesn't depend on the view shape
+                // as long as the switching button is present.
+                "base".into() => (c::Point { x: 0.0, y: 0.0 }, view.clone()),
+                "locked".into() => (c::Point { x: 0.0, y: 0.0 }, view),
+            },
+            purpose: ContentPurpose::Normal,
+        };
+
+        // Basic cycle
+        layout.apply_view_transition(&switch);
+        assert_eq!(&layout.current_view, "locked");
+        layout.apply_view_transition(&switch);
+        assert_eq!(&layout.current_view, "locked");
+        layout.apply_view_transition(&submit);
+        assert_eq!(&layout.current_view, "locked");
+        layout.apply_view_transition(&switch);
+        assert_eq!(&layout.current_view, "base");
+        layout.apply_view_transition(&switch);
+        // Unlatch
+        assert_eq!(&layout.current_view, "locked");
+        layout.apply_view_transition(&submit);
+        assert_eq!(&layout.current_view, "base");
+    }
+
+    #[test]
+    fn reverse_unlatch_layout() {
+        let switch = Action::LockView {
+            lock: "locked".into(),
+            unlock: "base".into(),
+            latches: true,
+            looks_locked_from: vec![],
+        };
+
+        let unswitch = Action::LockView {
+            lock: "locked".into(),
+            unlock: "unlocked".into(),
+            latches: false,
+            looks_locked_from: vec![],
+        };
+
+        let submit = Action::Erase;
+
+        let view = View::new(vec![(
+            0.0,
+            Row::new(vec![
+                (
+                    0.0,
+                    make_button_with_state(
+                        "switch".into(),
+                        make_state_with_action(switch.clone())
+                    ),
+                ),
+                (
+                    1.0,
+                    make_button_with_state(
+                        "submit".into(),
+                        make_state_with_action(submit.clone())
+                    ),
+                ),
+            ]),
+        )]);
+
+        let mut layout = Layout {
+            current_view: "base".into(),
+            view_latched: LatchedState::Not,
+            keymaps: Vec::new(),
+            kind: ArrangementKind::Base,
+            pressed_keys: HashSet::new(),
+            margins: Margins {
+                top: 0.0,
+                left: 0.0,
+                right: 0.0,
+                bottom: 0.0,
+            },
+            views: hashmap! {
+                // Both can use the same structure.
+                // Switching doesn't depend on the view shape
+                // as long as the switching button is present.
+                "base".into() => (c::Point { x: 0.0, y: 0.0 }, view.clone()),
+                "locked".into() => (c::Point { x: 0.0, y: 0.0 }, view.clone()),
+                "unlocked".into() => (c::Point { x: 0.0, y: 0.0 }, view),
+            },
+            purpose: ContentPurpose::Normal,
+        };
+
+        layout.apply_view_transition(&switch);
+        assert_eq!(&layout.current_view, "locked");
+        layout.apply_view_transition(&unswitch);
+        assert_eq!(&layout.current_view, "unlocked");
+    }
+
+    #[test]
+    fn latch_twopop_layout() {
+        let switch = Action::LockView {
+            lock: "locked".into(),
+            unlock: "base".into(),
+            latches: true,
+            looks_locked_from: vec![],
+        };
+
+        let switch_again = Action::LockView {
+            lock: "ĄĘ".into(),
+            unlock: "locked".into(),
+            latches: true,
+            looks_locked_from: vec![],
+        };
+
+        let submit = Action::Erase;
+
+        let view = View::new(vec![(
+            0.0,
+            Row::new(vec![
+                (
+                    0.0,
+                    make_button_with_state(
+                        "switch".into(),
+                        make_state_with_action(switch.clone())
+                    ),
+                ),
+                (
+                    1.0,
+                    make_button_with_state(
+                        "submit".into(),
+                        make_state_with_action(submit.clone())
+                    ),
+                ),
+            ]),
+        )]);
+
+        let mut layout = Layout {
+            current_view: "base".into(),
+            view_latched: LatchedState::Not,
+            keymaps: Vec::new(),
+            kind: ArrangementKind::Base,
+            pressed_keys: HashSet::new(),
+            margins: Margins {
+                top: 0.0,
+                left: 0.0,
+                right: 0.0,
+                bottom: 0.0,
+            },
+            views: hashmap! {
+                // All can use the same structure.
+                // Switching doesn't depend on the view shape
+                // as long as the switching button is present.
+                "base".into() => (c::Point { x: 0.0, y: 0.0 }, view.clone()),
+                "locked".into() => (c::Point { x: 0.0, y: 0.0 }, view.clone()),
+                "ĄĘ".into() => (c::Point { x: 0.0, y: 0.0 }, view),
+            },
+            purpose: ContentPurpose::Normal,
+        };
+
+        // Latch twice, then Ąto-unlatch across 2 levels
+        layout.apply_view_transition(&switch);
+        println!("{:?}", layout.view_latched);
+        assert_eq!(&layout.current_view, "locked");
+        layout.apply_view_transition(&switch_again);
+        println!("{:?}", layout.view_latched);
+        assert_eq!(&layout.current_view, "ĄĘ");
+        layout.apply_view_transition(&submit);
+        println!("{:?}", layout.view_latched);
+        assert_eq!(&layout.current_view, "base");
+    }
+
     #[test]
     fn check_centering() {
         //    A B
@@ -1193,6 +1441,7 @@ mod test {
         ]);
         let layout = Layout {
             current_view: String::new(),
+            view_latched: LatchedState::Not,
             keymaps: Vec::new(),
             kind: ArrangementKind::Base,
             pressed_keys: HashSet::new(),
@@ -1206,6 +1455,7 @@ mod test {
             views: hashmap! {
                 String::new() => (c::Point { x: 0.0, y: 0.0 }, view),
             },
+            purpose: ContentPurpose::Normal,
         };
         assert_eq!(
             layout.calculate_inner_size(),
